/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WPROBLEM_H
#define WPROBLEM_H

#include "waves.h"
#include "wObject.h"
#include <iostream>
#include <vector>
#include <memory>

using namespace tbox;

namespace waves
{

/**
 * @brief main wave Problem object
 */

class WAVES_API Problem : public fwk::wSharedObject
{
public:
    std::shared_ptr<MshData> msh;
#ifndef SWIG
    std::vector<std::shared_ptr<Source>> srcs;
    std::vector<std::shared_ptr<Medium>> media;
    std::vector<std::shared_ptr<Boundary>> bnds;
#endif
    Problem(std::shared_ptr<MshData> _msh);
    virtual ~Problem();

    void add(std::shared_ptr<Source> s) { srcs.push_back(s); }
    void add(std::shared_ptr<Medium> m) { media.push_back(m); }
    void add(std::shared_ptr<Boundary> b) { bnds.push_back(b); }

#ifndef SWIG
    void initElems();
    virtual void write(std::ostream &out) const override;
#endif
};
} // namespace waves

#endif //WPROBLEM_H
